-- Mod for Farming Simulator 17
-- Mod: Animal Notifications
-- Author: Corduroy
-- 		   contributions from MX11(added German translations, made sound path universal, added dryGrass to sheep feed section)

--change log:
	--version 1.1.0.0
		-- added Seasons compatibility
		-- refactored feed thresholds to increase accuracy
		-- changed manure notification to alert every 50K liters
		
	--version 1.0.0.0
		-- initial release

local modDesc = loadXMLFile("modDesc", g_currentModDirectory .. "modDesc.xml");

animalNotification = {};
animalNotification.version = getXMLString(modDesc, "modDesc.version");
animalNotification.modDirectory = g_currentModDirectory;

addModEventListener(animalNotification);

function animalNotification:loadMap()
	--animal bools and counters
	self.chickenIssues = 0;
	self.sheepIssues = 0;
	self.cowIssues = 0;
	self.pigIssues = 0;

	self.haveChickenIssues = false;
	self.haveSheepIssues = false;
	self.haveCowIssues = false;
	self.havePigIssues = false;

	self.numSheep = 0;
	self.numCows = 0;
	self.numPigs = 0;

	self.haveChickens = false;
	self.haveSheep = false;
	self.haveCows = false;
	self.havePigs = false;

	self.numSheepChanged = false;
	self.numCowsChanged = false;
	self.numPigsChanged = false;

	--animal births
	self.chickBirthCompare = 0;
	self.sheepBirthCompare = 0;
	self.cowBirthCompare = 0;
	self.pigBirthCompare = 0;
	self.birthTimerDuration = 0.25;

	--thresholds	
	local totalEggs = #g_currentMission.husbandries.chicken.pickupObjectsToActivate + g_currentMission.husbandries.chicken.numActivePickupObjects;
	self.eggThreshold = totalEggs - math.floor(totalEggs * 0.1);--90% "full"
	self.productivityThreshold = 40;--40%
	self.cleanlinessThreshold = 20;--20%
	self.manureThreshold = 50000;--notify every 50k liters
	self.manureFillLevel = math.floor(g_currentMission.husbandries.cow.manureFillLevel);
	self.currentManureThreshold = self.manureFillLevel - (self.manureFillLevel % self.manureThreshold) + self.manureThreshold;
	self.pigManureFillLevel = math.floor(g_currentMission.husbandries.pig.manureFillLevel);
	self.currentPigManureThreshold = self.pigManureFillLevel - (self.pigManureFillLevel % self.manureThreshold) + self.manureThreshold;
	self.feedWarnLevel = 0.2; --20 percent
	self.numFoodDays = 6; --seasons has 3 feed days, base game has 6 feed days
	self.sheepWaterThreshold = 0; 
	self.sheepGrassThreshold = 0;
	self.cowWaterThreshold = 0;
	self.cowStrawThreshold = 0;
	self.cowGrassThreshold = 0;
	self.silageThreshold = 0;
	self.tmrThreshold = 0;
	self.pigWaterThreshold = 0;
	self.pigStrawThreshold = 0;
	self.cornThreshold = 0;
	self.grainThreshold = 0;
	self.proteinThreshold = 0;
	self.rootThreshold = 0;

	--notify ({active message bool, [timerEndTime]})
	self.notify = {}
	--general
	self.notify.generalMessageActive = {false};
	--chickens
	self.notify.eggMessageActive = {false};
	self.notify.chickenBirthActive = {false};
	--sheep
	self.notify.productivityMessageSheepActive = {false};
	self.notify.cleanlinessMessageSheepActive = {false};
	self.notify.waterMessageSheepActive = {false};
	self.notify.grassMessageSheepActive = {false};
	self.notify.woolMessageActive = {false};
	self.notify.sheepBirthActive = {false};
	--pigs
	self.notify.productivityMessagePigActive = {false};
	self.notify.cleanlinessMessagePigActive = {false};
	self.notify.waterMessagePigActive = {false};
	self.notify.strawMessagePigActive = {false};
	self.notify.cornMessageActive = {false};
	self.notify.grainMessageActive = {false};
	self.notify.proteinMessageActive = {false};
	self.notify.root_cropsMessageActive = {false};
	self.notify.liquidManureMessagePigActive = {false};
	self.notify.manureMessagePigActive = {false};
	self.notify.pigBirthActive = {false};
	--cows
	self.notify.productivityMessageCowActive = {false};
	self.notify.cleanlinessMessageCowActive = {false};
	self.notify.waterMessageCowActive = {false};
	self.notify.strawMessageCowActive = {false};
	self.notify.grassMessageCowActive = {false};
	self.notify.silageMessageActive = {false};
	self.notify.tmrMessageActive = {false};
	self.notify.liquidManureMessageCowActive = {false};
	self.notify.manureMessageCowActive = {false};
	self.notify.cowBirthActive = {false};

	--notifications ({key, message})
	self.notifications = {};

	--snooze notifications
	self.snooze = false;
	self.snoozeTimerStart = 0;
	self.snoozeTimerEnd = 0;
	self.snoozeDuration = 5;

	--timeScale
	self.timeScale = g_currentMission.loadingScreen.missionInfo.timeScale;

	--set up sound
	self.previousActiveNotifications = 0;
	self.notificationSound = createSample("notify");
	loadSample(self.notificationSound, getAppBasePath().."data/maps/sounds/radio.wav", false);

	--20 minute initial animal update lag
	self.initializing = true;
	self.initialTime = g_currentMission.environment.dayTime;
	
	--seasons
	self.seasonsWeatherForecastActive = false;
	self.isWinter = false;
end;

function animalNotification:makeNotification(issue, animal, numAnimals)
	local message = "";

	if issue == "general" then
		message = g_i18n:getText("AN_GENERAL");

	elseif issue == "egg" then
		message = g_i18n:getText("AN_COLLECTEGGS");

	elseif issue == "wool" then
		message = g_i18n:getText("AN_WOOLPALLET");

	elseif issue == "birth" then
		message = string.format(g_i18n:getText("AN_ANIMALBORN"), string.lower(g_i18n:getText("shopItem_"..animal)));

	elseif issue == "cleanliness" then
		message = string.format(g_i18n:getText("AN_CLEANAREA"), string.lower(g_i18n:getText("shopItem_"..animal)));

	elseif issue == "productivity" then
		if g_seasons ~= nil then
			message = string.format(g_i18n:getText("AN_HEALTHLOW"), g_i18n:getText("shopItem_"..animal));
		else
			message = string.format(g_i18n:getText("AN_PRODUCTIVITYLOW"), g_i18n:getText("shopItem_"..animal));
		end;

	elseif issue == "slurry" then
		message = string.format(g_i18n:getText("AN_SLURRYTANKFULL"), string.lower(g_i18n:getText("shopItem_"..animal)));

	elseif issue == "manure" then
		message = string.format(g_i18n:getText("AN_MANUREFULL"), string.lower(g_i18n:getText("shopItem_"..animal)), numAnimals);--numAnimals is amount of manure in this case

	else
		if numAnimals == 1 then
			message = string.format(g_i18n:getText("AN_ANIMALNEEDS1"), string.lower(g_i18n:getText("shopItem_"..animal)), issue);
			
		elseif animal == "sheep" then
			message = string.format(g_i18n:getText("AN_ANIMALNEEDS2"), string.lower(g_i18n:getText("shopItem_"..animal)), issue);
		else
			message = string.format(g_i18n:getText("AN_ANIMALNEEDS3"), string.lower(g_i18n:getText("shopItem_"..animal)), issue);
		end;
	end;

	return(message);
end;

function animalNotification:checkChickens()
	local newAnimalThreshold = g_currentMission.husbandries.chicken.newAnimalPercentage;

	--eggs
	if math.floor(g_currentMission.husbandries.chicken.numActivePickupObjects) >= self.eggThreshold then
		if self.notify.eggMessageActive[1] == false then
			local message = animalNotification:makeNotification("egg", nil, nil);
			table.insert(self.notifications, {"egg", message});
			self.notify.eggMessageActive[1] = true;
			self.chickenIssues = self.chickenIssues + 1;
		end;
	elseif self.notify.eggMessageActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "egg" then
				table.remove(self.notifications, k);
				self.notify.eggMessageActive[1] = false;
				self.chickenIssues = self.chickenIssues - 1;
			end;
		end;
	end;

	--births
	--for future animal functions
	if newAnimalThreshold ~= nill and self.chickBirthCompare - newAnimalThreshold > 0 then
		if self.notify.chickenBirthActive[1] == false then
			local message = animalNotification:makeNotification("birth", "chicken", nil);
			table.insert(self.notifications, {"chickenBirth", message});
			self.notify.chickenBirthActive[1] = true;
			--set timer
			if self.notify.chickenBirthActive[2] == nil then
				self.notify.chickenBirthActive[3] = g_currentMission.environment.realHourTimer/1000/60;
				self.notify.chickenBirthActive[2] = (self.notify.chickenBirthActive[3] - self.birthTimerDuration) % 60; --timerEnd = currentTime + duration * timeScale
			end;
			self.chickenIssues = self.chickenIssues + 1;
		end;
	elseif self.notify.chickenBirthActive[1] then
		if self.timeScaleChanged then
			self.notify.chickenBirthActive[2] = (self.notify.chickenBirthActive[3] - self.birthTimerDuration) % 60; --recalculate endTime if timeScale changes
		end;
		if g_currentMission.environment.realHourTimer/1000/60 < self.notify.chickenBirthActive[2] then --is currentTime > timerEnd
			for k, v in pairs(self.notifications) do
				if v[1] == "chickenBirth" then
					table.remove(self.notifications, k);
					self.notify.chickenBirthActive[1] = false;
					self.notify.chickenBirthActive[2] = nil;
					self.notify.chickenBirthActive[3] = nil;
				end;
			end;
			self.chickenIssues = self.chickenIssues - 1;
		end;
	end;
	self.chickBirthCompare = newAnimalThreshold;

	self.havechickenIssues = self.chickenIssues > 0;
end;

function animalNotification:checkSheep()
	local newAnimalThreshold = g_currentMission.husbandries.sheep.newAnimalPercentage;
	--sheep thresholds
	--total fill volume is calculated using the number of animals (min of 15) * the amount of food needed per day (foodPerDay/strawPerDay/waterPerDay) * feed percentage (from animal menu) * number of days (3 or 6)
	--base game troughs contain 6 days of food.
	--seasons troughs contain 3 days.
	local sheep = g_currentMission.husbandries.sheep.animalDesc;
	local maxFill = 0;
	local numSheepToFeed = self.numSheep;
	--min 15 animals
	if numSheepToFeed < 15 then
		numSheepToFeed = 15;
	end;
	if self.numSheepChanged then
		--sheepWater threshold
		maxFill = sheep.waterPerDay * numSheepToFeed * self.numFoodDays;
		self.sheepWaterThreshold = maxFill * self.feedWarnLevel;
		--sheepGrass threshold
		maxFill = sheep.foodPerDay * numSheepToFeed * self.numFoodDays;
		self.sheepGrassThreshold = maxFill * self.feedWarnLevel;
	end;

	--check as long as the problem exists
	--wool
	local pallet = g_currentMission.husbandries.sheep.currentPallet;
	if pallet ~= nil then	
		if math.floor(pallet.fillLevel) >= pallet.capacity then
			if self.notify.woolMessageActive[1] == false then
				local message = animalNotification:makeNotification("wool", nil, nil);
				table.insert(self.notifications, {"wool", message});
				self.notify.woolMessageActive[1] = true;
				self.sheepIssues = self.sheepIssues + 1;
			end;
		elseif self.notify.woolMessageActive[1] then
			for k, v in pairs(self.notifications) do
				if v[1] == "wool" then
					table.remove(self.notifications, k);
					self.notify.woolMessageActive[1] = false;
					self.sheepIssues = self.sheepIssues - 1;
				end;
			end;
		end;
	end;

	--births
	if newAnimalThreshold ~= nill and self.sheepBirthCompare - newAnimalThreshold > 0 then
		if self.notify.sheepBirthActive[1] == false then
			local message = animalNotification:makeNotification("birth", "sheep", nil);
			table.insert(self.notifications, {"sheepBirth", message});
			self.notify.sheepBirthActive[1] = true;
			--set timer
			if self.notify.sheepBirthActive[2] == nil then
				self.notify.sheepBirthActive[3] = g_currentMission.environment.realHourTimer/1000/60; --time start
				self.notify.sheepBirthActive[2] = (self.notify.sheepBirthActive[3] - self.birthTimerDuration) % 60; --timerEnd = currentTime + duration * timeScale
			end;
			self.sheepIssues = self.sheepIssues + 1;
		end;
	elseif self.notify.sheepBirthActive[1] then
		if self.timeScaleChanged then
			self.notify.sheepBirthActive[2] = (self.notify.sheepBirthActive[3] - self.birthTimerDuration) % 60;--recalculate endTime if timeScale changes
		end;
		if g_currentMission.environment.realHourTimer/1000/60 < self.notify.sheepBirthActive[2] then --is currentTime > timerEnd
			for k, v in pairs(self.notifications) do
				if v[1] == "sheepBirth" then
					table.remove(self.notifications, k);
					self.notify.sheepBirthActive[1] = false;
					self.notify.sheepBirthActive[2] = nil;
					self.notify.sheepBirthActive[3] = nil;
				end;
			end;
			self.sheepIssues = self.sheepIssues - 1;
		end;
	end;
	self.sheepBirthCompare = newAnimalThreshold;

	--only check if sheep are owned
	--productivity
	local productivity;
	if self.haveSheep then
		if g_seasons ~= nil then
			productivity = g_currentMission.husbandries.sheep.averageProduction * 100;
		else
			productivity = g_currentMission.husbandries.sheep.productivity * 100;
		end;
	end;
-- ######################################################################################################################################################
	if self.haveSheep and self.initializing == false and math.floor(productivity) < self.productivityThreshold and math.floor(productivity) > 0 then
-- ######################################################################################################################################################
		if self.notify.productivityMessageSheepActive[1] == false then
			local message = animalNotification:makeNotification("productivity", "sheep", nil);
			table.insert(self.notifications, {"sheepProductivity", message});
			self.notify.productivityMessageSheepActive[1] = true;
			self.sheepIssues = self.sheepIssues + 1;
		end;
	elseif self.haveSheep == false or self.notify.productivityMessageSheepActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "sheepProductivity" then
				table.remove(self.notifications, k);
				self.notify.productivityMessageSheepActive[1] = false;
				self.sheepIssues = self.sheepIssues - 1;
			end;
		end;
	end;

	--cleanliness
	if self.haveSheep and math.floor(g_currentMission.husbandries.sheep.cleanlinessFactor * 100) < self.cleanlinessThreshold then
		if self.notify.cleanlinessMessageSheepActive[1] == false then
			local message = animalNotification:makeNotification("cleanliness", "sheep", nil);
			table.insert(self.notifications, {"sheepCleanliness", message});
			self.notify.cleanlinessMessageSheepActive[1] = true;
			self.sheepIssues = self.sheepIssues + 1;
		end;
	elseif self.haveSheep == false or self.notify.cleanlinessMessageSheepActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "sheepCleanliness" then
				table.remove(self.notifications, k);
				self.notify.cleanlinessMessageSheepActive[1] = false;
				self.sheepIssues = self.sheepIssues - 1;
			end;
		end;
	end;

	--water
	if self.haveSheep and math.floor(g_currentMission.husbandries.sheep:getFillLevel(FillUtil.FILLTYPE_WATER)) < self.sheepWaterThreshold then
		if self.notify.waterMessageSheepActive[1] == false then
			local message = animalNotification:makeNotification("water", "sheep", g_currentMission.husbandries.sheep.totalNumAnimals);
			table.insert(self.notifications, {"sheepWater", message});
			self.notify.waterMessageSheepActive[1] = true;
			self.sheepIssues = self.sheepIssues + 1;
		elseif self.numSheepChanged then 
			local message = animalNotification:makeNotification("water", "sheep", self.numSheep);
			for k, v in pairs(self.notifications) do
				if v[1] == "sheepWater" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.haveSheep == false or self.notify.waterMessageSheepActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "sheepWater" then
				table.remove(self.notifications, k);
				self.notify.waterMessageSheepActive[1] = false;
				self.sheepIssues = self.sheepIssues - 1;
			end;
		end;
	end;

		--grass
	if self.haveSheep and math.floor(g_currentMission.husbandries.sheep:getFillLevel(FillUtil.FILLTYPE_GRASS_WINDROW) + g_currentMission.husbandries.sheep:getFillLevel(FillUtil.FILLTYPE_DRYGRASS_WINDROW)) < self.sheepGrassThreshold then
		if self.notify.grassMessageSheepActive[1] == false then
			local message = animalNotification:makeNotification("grass", "sheep", g_currentMission.husbandries.sheep.totalNumAnimals);
			table.insert(self.notifications, {"sheepGrass", message});
			self.notify.grassMessageSheepActive[1] = true;
			self.sheepIssues = self.sheepIssues + 1;
		elseif self.numSheepChanged then 
			local message = animalNotification:makeNotification("grass", "sheep", self.numSheep);
			for k, v in pairs(self.notifications) do
				if v[1] == "sheepGrass" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.haveSheep == false or self.notify.grassMessageSheepActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "sheepGrass" then
				table.remove(self.notifications, k);
				self.notify.grassMessageSheepActive[1] = false;
				self.sheepIssues = self.sheepIssues - 1;
			end;
		end;
	end;

	self.haveSheepIssues = self.sheepIssues > 0;
end;

function animalNotification:checkCows()
	local newAnimalThreshold = g_currentMission.husbandries.cow.newAnimalPercentage;
	--cow thresholds
	--total fill volume is calculated using the number of animals (min of 15) * the amount of food needed per day (foodPerDay/strawPerDay/waterPerDay) * feed percentage (from animal menu) * number of days (3 or 6)
	--base game troughs contain 6 days of food.
	--seasons troughs contain 3 days.
	local cow = g_currentMission.husbandries.cow.animalDesc;
	local maxFill = 0;
	local grassPercent = 0.2;--20%
	local basePercent = 0.5;--50%
	local tmrPercent = 0.3;--30%
	local numCowsToFeed = self.numCows;
	--min 15 animals
	if numCowsToFeed < 15 then
		numCowsToFeed = 15;
	end;

	if self.numCowsChanged then
		--cowWater threshold
		maxFill = cow.waterPerDay * numCowsToFeed * self.numFoodDays;
		self.cowWaterThreshold = maxFill * self.feedWarnLevel;

		--cowStraw threshold
		maxFill = cow.strawPerDay * numCowsToFeed * self.numFoodDays;
		self.cowStrawThreshold = maxFill * self.feedWarnLevel;

		--cowGrass threshold
		maxFill = cow.foodPerDay * numCowsToFeed * grassPercent * self.numFoodDays;
		self.cowGrassThreshold = maxFill * self.feedWarnLevel;
		
		--silage threshold
		maxFill = cow.foodPerDay * numCowsToFeed * basePercent * self.numFoodDays;
		self.silageThreshold = maxFill * self.feedWarnLevel;

		--tmr threshold
		maxFill = cow.foodPerDay * numCowsToFeed * tmrPercent * self.numFoodDays;
		self.tmrThreshold = maxFill * self.feedWarnLevel;
	end;

	--check as long as the problem exists
	--liquidManure
	local liquidManure = g_currentMission.husbandries.cow.liquidManureTrigger;
	if math.floor(liquidManure.fillLevel) > liquidManure.capacity then
		if self.notify.liquidManureMessageCowActive[1] == false then
			local message = animalNotification:makeNotification("slurry", "cow", nil);
			table.insert(self.notifications, {"cowSlurry", message});
			self.notify.liquidManureMessageCowActive[1] = true;
			self.cowIssues = self.cowIssues + 1;
		end;
	elseif self.notify.liquidManureMessageCowActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "cowSlurry" then
				table.remove(self.notifications, k);
				self.notify.liquidManureMessageCowActive[1] = false;
				self.cowIssues = self.cowIssues - 1;
			end;
		end;
	end;

	--manure
	local fillLevel = math.floor(g_currentMission.husbandries.cow.manureFillLevel);
	--if manure is taken out of bin...
	if fillLevel < self.manureFillLevel then
		--...recalc threshold
		self.currentManureThreshold = fillLevel - (fillLevel % self.manureThreshold) + self.manureThreshold;
		self.manureFillLevel = fillLevel;
	end;
	--check fillLevel against threshold
	if fillLevel > self.currentManureThreshold then		
		if self.notify.manureMessageCowActive[1] == false then
			local message = animalNotification:makeNotification("manure", "cow", self.currentManureThreshold);
			table.insert(self.notifications, {"cowManure", message});
			self.notify.manureMessageCowActive[1] = true;
			--set timer
			if self.notify.manureMessageCowActive[2] == nil then
				self.notify.manureMessageCowActive[3] = g_currentMission.environment.realHourTimer/1000/60;
				self.notify.manureMessageCowActive[2] = (self.notify.manureMessageCowActive[3] - self.birthTimerDuration) % 60; --timerEnd = currentTime + duration * timeScale
			end;
			self.cowIssues = self.cowIssues + 1;			
		end;
		--increase threshold by 50
		self.currentManureThreshold = self.currentManureThreshold + self.manureThreshold;
		self.manureFillLevel = fillLevel;
	elseif self.notify.manureMessageCowActive[1] then
		if self.timeScaleChanged then
			self.notify.manureMessageCowActive[2] = (self.notify.manureMessageCowActive[3] - self.birthTimerDuration) % 60; --recalculate endTime if timeScale changes
		end;
		if g_currentMission.environment.realHourTimer/1000/60 < self.notify.manureMessageCowActive[2] then --is currentTime > timerEnd
			for k, v in pairs(self.notifications) do
				if v[1] == "cowManure" then
					table.remove(self.notifications, k);
					self.notify.manureMessageCowActive[1] = false;
					self.notify.manureMessageCowActive[2] = nil;
					self.notify.manureMessageCowActive[3] = nil;
				end;
			end;
			self.cowIssues = self.cowIssues - 1;
		end;
	end;

	--births
	if newAnimalThreshold ~= nill and self.cowBirthCompare - newAnimalThreshold > 0 then
		if self.notify.cowBirthActive[1] == false then
			local message = animalNotification:makeNotification("birth", "cow", nil);
			table.insert(self.notifications, {"cowBirth", message});
			self.notify.cowBirthActive[1] = true;
			--set timer
			if self.notify.cowBirthActive[2] == nil then
				self.notify.cowBirthActive[3] = g_currentMission.environment.realHourTimer/1000/60;
				self.notify.cowBirthActive[2] = (self.notify.cowBirthActive[3] - self.birthTimerDuration) % 60; --timerEnd = currentTime + duration * timeScale
			end;
			self.cowIssues = self.cowIssues + 1;
		end;
	elseif self.notify.cowBirthActive[1] then
		if self.timeScaleChanged then
			self.notify.cowBirthActive[2] = (self.notify.cowBirthActive[3] - self.birthTimerDuration) % 60; --recalculate endTime if timeScale changes
		end;
		if g_currentMission.environment.realHourTimer/1000/60 < self.notify.cowBirthActive[2] then --is currentTime > timerEnd
			for k, v in pairs(self.notifications) do
				if v[1] == "cowBirth" then
					table.remove(self.notifications, k);
					self.notify.cowBirthActive[1] = false;
					self.notify.cowBirthActive[2] = nil;
					self.notify.cowBirthActive[3] = nil;
				end;
			end;
			self.cowIssues = self.cowIssues - 1;
		end;
	end;
	self.cowBirthCompare = newAnimalThreshold;

	--only check if cows are owned
	--productivity
	local productivity;
	if self.haveCows then		
		if g_seasons ~= nil then
			productivity = g_currentMission.husbandries.cow.averageProduction * 100;
		else
			productivity = g_currentMission.husbandries.cow.productivity * 100;
		end;
	end;
	if self.haveCows and self.initializing == false and math.floor(productivity) < self.productivityThreshold and math.floor(productivity) > 0 then
		if self.notify.productivityMessageCowActive[1] == false then
			local message = animalNotification:makeNotification("productivity", "cow", nil);
			table.insert(self.notifications, {"cowProductivity", message});
			self.notify.productivityMessageCowActive[1] = true;
			self.cowIssues = self.cowIssues + 1;
		end;
	elseif self.haveCows == false or self.notify.productivityMessageCowActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "cowProductivity" then
				table.remove(self.notifications, k);
				self.notify.productivityMessageCowActive[1] = false;
				self.cowIssues = self.cowIssues - 1;
			end;
		end;
	end;

	--cleanliness
	if self.haveCows and math.floor(g_currentMission.husbandries.cow.cleanlinessFactor * 100) < self.cleanlinessThreshold then
		if self.notify.cleanlinessMessageCowActive[1] == false then
			local message = animalNotification:makeNotification("cleanliness", "cow", nil);
			table.insert(self.notifications, {"cowCleanliness", message});
			self.notify.cleanlinessMessageCowActive[1] = true;
			self.cowIssues = self.cowIssues + 1;
		end;
	elseif self.haveCows == false or self.notify.cleanlinessMessageCowActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "cowCleanliness" then
				table.remove(self.notifications, k);
				self.notify.cleanlinessMessageCowActive[1] = false;
				self.cowIssues = self.cowIssues - 1;
			end;
		end;
	end;

	--water
	if self.haveCows and math.floor(g_currentMission.husbandries.cow:getFillLevel(FillUtil.FILLTYPE_WATER)) < self.cowWaterThreshold then
		if self.notify.waterMessageCowActive[1] == false then
			local message = animalNotification:makeNotification("water", "cow", g_currentMission.husbandries.cow.totalNumAnimals);
			table.insert(self.notifications, {"cowWater", message});
			self.notify.waterMessageCowActive[1] = true;
			self.cowIssues = self.cowIssues + 1;
		elseif self.numCowsChanged then 
			--update message to refelct the correct number of animals
			local message = animalNotification:makeNotification("water", "cow", self.numCows);
			for k, v in pairs(self.notifications) do
				if v[1] == "cowWater" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.haveCows == false or self.notify.waterMessageCowActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "cowWater" then
				table.remove(self.notifications, k);
				self.notify.waterMessageCowActive[1] = false;
				self.cowIssues = self.cowIssues - 1;
			end;
		end;
	end;

	--straw
	if self.haveCows and math.floor(g_currentMission.husbandries.cow:getFillLevel(FillUtil.FILLTYPE_STRAW)) < self.cowStrawThreshold then
		if self.notify.strawMessageCowActive[1] == false then
			local message = animalNotification:makeNotification("straw", "cow", g_currentMission.husbandries.cow.totalNumAnimals);
			table.insert(self.notifications, {"cowStraw", message});
			self.notify.strawMessageCowActive[1] = true;
			self.cowIssues = self.cowIssues + 1;
		elseif self.numCowsChanged then 
			--update message to refelct the correct number of animals
			local message = animalNotification:makeNotification("straw", "cow", self.numCows);
			for k, v in pairs(self.notifications) do
				if v[1] == "cowStraw" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.haveCows == false or self.notify.strawMessageCowActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "cowStraw" then
				table.remove(self.notifications, k);
				self.notify.strawMessageCowActive[1] = false;
				self.cowIssues = self.cowIssues - 1;
			end;
		end;
	end;

	--grass
	if self.haveCows and self.isWinter == false and math.floor(g_currentMission.husbandries.cow:getFillLevel(FillUtil.FILLTYPE_GRASS_WINDROW)) < self.cowGrassThreshold then
		if self.notify.grassMessageCowActive[1] == false then
			local message = animalNotification:makeNotification("grass", "cow", g_currentMission.husbandries.cow.totalNumAnimals);
			table.insert(self.notifications, {"cowGrass", message});
			self.notify.grassMessageCowActive[1] = true;
			self.cowIssues = self.cowIssues + 1;
		elseif self.numCowsChanged then 
			--update message to refelct the correct number of animals
			local message = animalNotification:makeNotification("grass", "cow", self.numCows);
			for k, v in pairs(self.notifications) do
				if v[1] == "cowGrass" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.haveCows == false or self.notify.grassMessageCowActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "cowGrass" then
				table.remove(self.notifications, k);
				self.notify.grassMessageCowActive[1] = false;
				self.cowIssues = self.cowIssues - 1;
			end;
		end;
	end;

	--silage
	if self.haveCows and math.floor(g_currentMission.husbandries.cow:getFillLevel(FillUtil.FILLTYPE_DRYGRASS_WINDROW) + g_currentMission.husbandries.cow:getFillLevel(FillUtil.FILLTYPE_SILAGE)) < self.silageThreshold then
		if self.notify.silageMessageActive[1] == false then
			local message = animalNotification:makeNotification("silage", "cow", g_currentMission.husbandries.cow.totalNumAnimals);
			table.insert(self.notifications, {"cowSilage", message});
			self.notify.silageMessageActive[1] = true;
			self.cowIssues = self.cowIssues + 1;
		elseif self.numCowsChanged then 
			--update message to refelct the correct number of animals
			local message = animalNotification:makeNotification("silage", "cow", self.numCows);
			for k, v in pairs(self.notifications) do
				if v[1] == "cowSilage" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.haveCows == false or self.notify.silageMessageActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "cowSilage" then
				table.remove(self.notifications, k);
				self.notify.silageMessageActive[1] = false;
				self.cowIssues = self.cowIssues - 1;
			end;
		end;
	end;

	--tmr
	if self.haveCows and math.floor(g_currentMission.husbandries.cow:getFillLevel(FillUtil.FILLTYPE_POWERFOOD)) < self.tmrThreshold then
		if self.notify.tmrMessageActive[1] == false then
			local message = animalNotification:makeNotification("Total Mixed Ration", "cow", g_currentMission.husbandries.cow.totalNumAnimals);
			table.insert(self.notifications, {"cowTMR", message});
			self.notify.tmrMessageActive[1] = true;
			self.cowIssues = self.cowIssues + 1;
		elseif self.numCowsChanged then 
			--update message to refelct the correct number of animals
			local message = animalNotification:makeNotification("Total Mixed Ration", "cow", self.numCows);
			for k, v in pairs(self.notifications) do
				if v[1] == "cowTMR" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.haveCows == false or self.notify.tmrMessageActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "cowTMR" then
				table.remove(self.notifications, k);
				self.notify.tmrMessageActive[1] = false;
				self.cowIssues = self.cowIssues - 1;
			end;
		end;
	end;

	self.haveCowIssues = self.cowIssues > 0;
end;

function animalNotification:checkPigs()
	local newAnimalThreshold = g_currentMission.husbandries.pig.newAnimalPercentage;
	--pig thresholds
	--total fill volume is calculated using the number of animals (min of 15) * the amount of food needed per day (foodPerDay/strawPerDay/waterPerDay) * feed percentage (from animal menu) * number of days (3 or 6)
	--base game troughs contain 6 days of food.
	--seasons troughs contain 3 days.
	local pig = g_currentMission.husbandries.pig.animalDesc;
	local maxFill = 0;
	local cornPercent = 0.5;--50%
	local grainPercent = 0.25;--25%
	local protienPercent = 0.2;--20%
	local rootPercent = 0.05;--5%
	local numPigsToFeed = self.numCows;
	--min 15 animals
	if numPigsToFeed < 15 then
		numPigsToFeed = 15;
	end;

	if self.numPigsChanged then
		--pigWater threshold
		maxFill = pig.waterPerDay * numPigsToFeed * self.numFoodDays;
		self.pigWaterThreshold = maxFill * self.feedWarnLevel;

		--pigStraw threshold
		maxFill = pig.strawPerDay * numPigsToFeed * self.numFoodDays;
		self.pigStrawThreshold = maxFill * self.feedWarnLevel;

		--corn threshold
		maxFill = pig.foodPerDay * numPigsToFeed * cornPercent * self.numFoodDays;
		self.cornThreshold = maxFill * self.feedWarnLevel;

		--grain threshold
		maxFill = pig.foodPerDay * numPigsToFeed * grainPercent * self.numFoodDays;
		self.grainThreshold = maxFill * self.feedWarnLevel;

		--protein threshold
		maxFill = pig.foodPerDay * numPigsToFeed * protienPercent * self.numFoodDays;
		self.proteinThreshold = maxFill * self.feedWarnLevel;

		--roots threshold
		maxFill = pig.foodPerDay * numPigsToFeed * rootPercent * self.numFoodDays;
		self.rootThreshold = maxFill * self.feedWarnLevel;
	end;

	--check as long as the problem exists
	--liquidManure
	local liquidManure = g_currentMission.husbandries.pig.liquidManureTrigger;
	if math.floor(liquidManure.fillLevel) > liquidManure.capacity then
		if self.notify.liquidManureMessagePigActive[1] == false then
			local message = animalNotification:makeNotification("slurry", "pig", nil);
			table.insert(self.notifications, {"pigSlurry", message});
			self.notify.liquidManureMessagePigActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		end;
	elseif self.notify.liquidManureMessagePigActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigSlurry" then
				table.remove(self.notifications, k);
				self.notify.liquidManureMessagePigActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	--manure
	local fillLevel = math.floor(g_currentMission.husbandries.pig.manureFillLevel);
	--if manure is taken out of bin...
	if fillLevel < self.pigManureFillLevel then
		--...recalc threshold
		self.currentPigManureThreshold = fillLevel - (fillLevel % self.manureThreshold) + self.manureThreshold;
		self.pigManureFillLevel = fillLevel;
	end;
	--check fillLevel against threshold
	if fillLevel > self.currentPigManureThreshold then
		if self.notify.manureMessagePigActive[1] == false then
			local message = animalNotification:makeNotification("manure", "pig", self.currentPigManureThreshold);
			table.insert(self.notifications, {"pigManure", message});
			self.notify.manureMessagePigActive[1] = true;
			--set timer
			if self.notify.manureMessagePigActive[2] == nil then
				self.notify.manureMessagePigActive[3] = g_currentMission.environment.realHourTimer/1000/60;
				self.notify.manureMessagePigActive[2] = (self.notify.manureMessagePigActive[3] - self.birthTimerDuration) % 60; --timerEnd = currentTime + duration * timeScale
			end;
			self.pigIssues = self.pigIssues + 1;
		end;
		--increase threshold by 50
		self.currentPigManureThreshold = self.currentPigManureThreshold + self.manureThreshold;
		self.pigManureFillLevel = fillLevel;
	elseif self.notify.manureMessagePigActive[1] then
		if self.timeScaleChanged then
			self.notify.manureMessagePigActive[2] = (self.notify.manureMessagePigActive[3] - self.birthTimerDuration) % 60; --recalculate endTime if timeScale changes
		end;
		if g_currentMission.environment.realHourTimer/1000/60 < self.notify.manureMessagePigActive[2] then --is currentTime > timerEnd
			for k, v in pairs(self.notifications) do
				if v[1] == "pigManure" then
					table.remove(self.notifications, k);
					self.notify.manureMessagePigActive[1] = false;
					self.notify.manureMessagePigActive[2] = nil;
					self.notify.manureMessagePigActive[3] = nil;
				end;
			end;
			self.pigIssues = self.pigIssues - 1;
		end;
	end;

	--births
	if newAnimalThreshold ~= nill and self.pigBirthCompare - newAnimalThreshold > 0 then
		if self.notify.pigBirthActive[1] == false then
			local message = animalNotification:makeNotification("birth", "pig", nil);
			table.insert(self.notifications, {"pigBirth", message});
			self.notify.pigBirthActive[1] = true;
			--set timer
			if self.notify.pigBirthActive[2] == nil then
				self.notify.pigBirthActive[3] = g_currentMission.environment.realHourTimer/1000/60;
				self.notify.pigBirthActive[2] = (self.notify.pigBirthActive[3] - self.birthTimerDuration) % 60; --timerEnd = currentTime + duration * timeScale
			end;
			self.pigIssues = self.pigIssues + 1;
		end;
	elseif self.notify.pigBirthActive[1] then
		if self.timeScaleChanged then
			self.notify.pigBirthActive[2] = (self.notify.pigBirthActive[3] - self.birthTimerDuration) % 60; --recalculate endTime if timeScale changes
		end;
		if g_currentMission.environment.realHourTimer/1000/60 < self.notify.pigBirthActive[2] then --is currentTime < timerEnd
			for k, v in pairs(self.notifications) do
				if v[1] == "pigBirth" then
					table.remove(self.notifications, k);
					self.notify.pigBirthActive[1] = false;
					self.notify.pigBirthActive[2] = nil;
					self.notify.pigBirthActive[3] = nil;
				end;
			end;
			self.pigIssues = self.pigIssues - 1;
		end;
	end;
	self.pigBirthCompare = newAnimalThreshold;

	--only check if pigs are owned
	--productivity
	local productivity;
	if self.havePigs then
		if g_seasons ~= nil then
			productivity = g_currentMission.husbandries.pig.averageProduction * 100;
		else
			productivity = g_currentMission.husbandries.pig.productivity * 100;
		end;
	end;
	if self.havePigs and self.initializing == false and math.floor(productivity) < self.productivityThreshold and math.floor(productivity) > 0 then
		if self.notify.productivityMessagePigActive[1] == false then
			local message = animalNotification:makeNotification("productivity", "pig", nil);
			table.insert(self.notifications, {"pigProductivity", message});
			self.notify.productivityMessagePigActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		end;
	--turn off if pigs are sold or the productivity raises above threshold
	elseif self.havePigs == false or self.notify.productivityMessagePigActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigProductivity" then
				table.remove(self.notifications, k);
				self.notify.productivityMessagePigActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	--cleanliness
	if self.havePigs and math.floor(g_currentMission.husbandries.pig.cleanlinessFactor * 100) < self.cleanlinessThreshold then
		if self.notify.cleanlinessMessagePigActive[1] == false then
			local message = animalNotification:makeNotification("cleanliness", "pig", nil);
			table.insert(self.notifications, {"pigCleanliness", message});
			self.notify.cleanlinessMessagePigActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		end;
	elseif self.havePigs == false or self.notify.cleanlinessMessagePigActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigCleanliness" then
				table.remove(self.notifications, k);
				self.notify.cleanlinessMessagePigActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	--water
	if self.havePigs and math.floor(g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_WATER)) < self.pigWaterThreshold then
		if self.notify.waterMessagePigActive[1] == false then
			local message = animalNotification:makeNotification("water", "pig", self.numPigs);
			table.insert(self.notifications, {"pigWater", message});
			self.notify.waterMessagePigActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		elseif self.numPigsChanged then 
			local message = animalNotification:makeNotification("water", "pig", self.numPigs);
			for k, v in pairs(self.notifications) do
				if v[1] == "pigWater" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.havePigs == false or self.notify.waterMessagePigActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigWater" then
				table.remove(self.notifications, k);
				self.notify.waterMessagePigActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	--straw
	if self.havePigs and math.floor(g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_STRAW)) < self.pigStrawThreshold then
		if self.notify.strawMessagePigActive[1] == false then
			local message = animalNotification:makeNotification("straw", "pig", self.numPigs);
			table.insert(self.notifications, {"pigStraw", message});
			self.notify.strawMessagePigActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		elseif self.numPigsChanged then 
			local message = animalNotification:makeNotification("straw", "pig", self.numPigs);
			for k, v in pairs(self.notifications) do
				if v[1] == "pigStraw" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.havePigs == false or self.notify.strawMessagePigActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigStraw" then
				table.remove(self.notifications, k);
				self.notify.strawMessagePigActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	--corn
	if self.havePigs and math.floor(g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_MAIZE)) < self.cornThreshold then
		if self.notify.cornMessageActive[1] == false then
			local message = animalNotification:makeNotification("corn", "pig", self.numPigs);
			table.insert(self.notifications, {"pigCorn", message});
			self.notify.cornMessageActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		elseif self.numPigsChanged then 
			local message = animalNotification:makeNotification("corn", "pig", self.numPigs);
			for k, v in pairs(self.notifications) do
				if v[1] == "pigCorn" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.havePigs == false or self.notify.cornMessageActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigCorn" then
				table.remove(self.notifications, k);
				self.notify.cornMessageActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	--grain
	if self.havePigs and math.floor(g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_WHEAT) + g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_BARLEY)) < self.grainThreshold then
		if self.notify.grainMessageActive[1] == false then
			local message = animalNotification:makeNotification("grain", "pig", self.numPigs);
			table.insert(self.notifications, {"pigGrain", message});
			self.notify.grainMessageActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		elseif self.numPigsChanged then 
			local message = animalNotification:makeNotification("grain", "pig", self.numPigs);
			for k, v in pairs(self.notifications) do
				if v[1] == "pigGrain" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.havePigs == false or self.notify.grainMessageActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigGrain" then
				table.remove(self.notifications, k);
				self.notify.grainMessageActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	--protein
	if self.havePigs and math.floor(g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_RAPE) + g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_SUNFLOWER) + g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_SOYBEAN)) < self.proteinThreshold then
		if self.notify.proteinMessageActive[1] == false then
			local message = animalNotification:makeNotification("protein", "pig", self.numPigs);
			table.insert(self.notifications, {"pigProtein", message});
			self.notify.proteinMessageActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		elseif self.numPigsChanged then 
			local message = animalNotification:makeNotification("protein", "pig", self.numPigs);
			for k, v in pairs(self.notifications) do
				if v[1] == "pigProtein" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.havePigs == false or self.notify.proteinMessageActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigProtein" then
				table.remove(self.notifications, k);
				self.notify.proteinMessageActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	--root crops
	if self.havePigs and math.floor(g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_POTATO) + g_currentMission.husbandries.pig:getFillLevel(FillUtil.FILLTYPE_SUGARBEET)) < self.rootThreshold then
		if self.notify.root_cropsMessageActive[1] == false then
			local message = animalNotification:makeNotification("root crops", "pig", self.numPigs);
			table.insert(self.notifications, {"pigRoots", message});
			self.notify.root_cropsMessageActive[1] = true;
			self.pigIssues = self.pigIssues + 1;
		elseif self.numPigsChanged then 
			local message = animalNotification:makeNotification("root crops", "pig", self.numPigs);
			for k, v in pairs(self.notifications) do
				if v[1] == "pigRoots" then
					v[2] = message;
				end;
			end;
		end;
	elseif self.havePigs == false or self.notify.root_cropsMessageActive[1] then
		for k, v in pairs(self.notifications) do
			if v[1] == "pigRoots" then
				table.remove(self.notifications, k);
				self.notify.root_cropsMessageActive[1] = false;
				self.pigIssues = self.pigIssues - 1;
			end;
		end;
	end;

	self.havePigIssues = self.pigIssues > 0;
end;

function animalNotification:keyEvent(unicode, sym, modifier, isDown)
end;

function animalNotification:mouseEvent(posX, posY, isDown, isUp, button)
end;

function animalNotification:deleteMap()
	for k, v in pairs(self.notifications) do
		 v = nil;
	end;

	for k, v in pairs(self.notify) do
		 v = nil;
	end;
end;

function animalNotification:update(dt)
	--check for current season and weather forecast overlay
	if g_seasons ~= nil then	
		local season = g_seasons.environment:currentSeason();
		self.isWinter = (season == 3);
		
		self.seasonsWeatherForecastActive = g_seasons.forecast.visible;
	end;

	--20 minute initial animal update lag
	if self.initializing then
		--number of feed days
		if g_seasons ~= nil then
			self.numFoodDays = 3;
		end; 
		
		self.initializing = g_currentMission.environment.dayTime > (self.initialTime + 1270000) % 86400000;-- 1270000 == 20 mins
	end;

	--timeScale
	if self.timeScale ~= g_currentMission.loadingScreen.missionInfo.timeScale then
		self.timeScaleChanged = true;
		self.timeScale = g_currentMission.loadingScreen.missionInfo.timeScale
	else
		self.timeScaleChanged = false;
	end;

	--chickens
	self.haveChickens = g_currentMission.husbandries.chicken.totalNumAnimals > 0;

	if self.haveChickens or self.haveChickenIssues then
		animalNotification:checkChickens();
	end;

	--sheep
	if self.numSheep ~= g_currentMission.husbandries.sheep.totalNumAnimals then
		self.numSheep = g_currentMission.husbandries.sheep.totalNumAnimals;
		self.numSheepChanged = true;
	else
		self.numSheepChanged = false;
	end;

	self.haveSheep = self.numSheep > 0;

	if self.haveSheep or self.haveSheepIssues then
		animalNotification:checkSheep();
	end;

	--cows
	if self.numCows ~= g_currentMission.husbandries.cow.totalNumAnimals then
		self.numCows = g_currentMission.husbandries.cow.totalNumAnimals;
		self.numCowsChanged = true;
	else
		self.numCowsChanged = false;
	end;
	
	self.haveCows = self.numCows > 0;
	
	if self.haveCows or self.haveCowIssues then
		animalNotification:checkCows();
	end;

	--pigs
	if self.numPigs ~= g_currentMission.husbandries.pig.totalNumAnimals then
		self.numPigs = g_currentMission.husbandries.pig.totalNumAnimals;
		self.numPigsChanged = true;
	else
		self.numPigsChanged = false;
	end;
	
	self.havePigs = self.numPigs > 0;
	
	if self.havePigs or self.havePigIssues then
		animalNotification:checkPigs();
	end;

	--snooze notifications
	if InputBinding.hasEvent(InputBinding.NOTE_DELAY) and self.snooze == false then
		self.snooze = true;
		self.snoozeTimerStart = g_currentMission.environment.realHourTimer/1000/60;
		self.snoozeTimerEnd = (self.snoozeTimerStart - self.snoozeDuration) % 60;
	end;
	if self.snooze then
		if self.timeScaleChanged then
			self.snoozeTimerEnd = (self.snoozeTimerStart - self.snoozeDuration) % 60; --recalculate if timeScale changes
		end;
		if g_currentMission.environment.realHourTimer/1000/60 < self.snoozeTimerEnd then
			self.snooze = false;
		end;
	end;
end;

function animalNotification:draw()
	local activeNotices = 0;
	if self.snooze == false then
		local fontSize = 0.015;
		local spacing = 0;
		local bgHeight = 0.03;
		local bgSpacing = 0;
		local posX;
		local posY;
		local bgPosX;
		local bgPosY;
		local bgWidth;
		if self.seasonsWeatherForecastActive then
			posX = g_seasons.forecast.forecastX + 0.0055;
			posY = g_seasons.forecast.forecastY - 0.02;
			bgPosX = g_seasons.forecast.forecastX;
			bgPosY = g_seasons.forecast.forecastY - bgHeight;
			bgWidth = g_seasons.forecast.forecastWidth;
		else
			posX = g_currentMission.infoBarBgOverlay.x + 0.0055;
			posY = g_currentMission.infoBarBgOverlay.y - 0.02;
			bgPosX = g_currentMission.infoBarBgOverlay.x;
			bgPosY = g_currentMission.infoBarBgOverlay.y - bgHeight;
			bgWidth = g_currentMission.infoBarBgOverlay.width;
		end;
		
		activeNotices = #self.notifications;
	
		setTextAlignment(RenderText.ALIGN_LEFT);

		if activeNotices > 9 then
			local message = animalNotification:makeNotification("general", nil, nil);
			local notificationBg = Overlay:new("notificationBg", Utils.getFilename("notifcationBg.dds", animalNotification.modDirectory), bgPosX, bgPosY - bgSpacing, bgWidth, bgHeight);
			notificationBg:render();
			renderText(posX, posY - spacing, fontSize, message);
		else
			for i = activeNotices, 1, -1 do
				if self.notifications[i][2] ~= nil then
					local notificationBg = Overlay:new("notificationBg", Utils.getFilename("notifcationBg.dds", animalNotification.modDirectory), bgPosX, bgPosY - bgSpacing, bgWidth, bgHeight);
					notificationBg:render();
					renderText(posX, posY - spacing, fontSize, self.notifications[i][2]);
				end;
				spacing = spacing + bgHeight;
				bgSpacing = bgSpacing + bgHeight;
			end;
		end;
	end;

	if activeNotices ~= self.previousActiveNotifications then
		if activeNotices > self.previousActiveNotifications then
			playSample(self.notificationSound, 1, 1, 0);
		end;
		self.previousActiveNotifications = activeNotices;
	end;
end;
